// locate & size final output
xoff = 200;
yoff = 200;
xyScale = 175;
zScale = 4;

twoPI = 2*Math.PI;
rad2deg = 180 / Math.PI;

keycodeA = 65;
keycodeB = 66;
keycodeC = 67;
keycodeD = 68;
keycodeJ = 74;
keycodeN = 78;
keycodeS = 83;
keycodeT = 84;
keycodeU = 85;
keycodeV = 86;
keycodeW = 87;
keycodeX = 88;
keycodeY = 89;
keycodeZ = 90;

KEYBOARD  = 1;
AUTOPILOT = 2;
g_iControl = KEYBOARD;

iKeySpeed = 0.1

function paint(){
  xyScale = mcControlPanel.sldScale.GetValue();
  alpha = mcControlPanel.sldTransparency.GetValue();

  // increment 3 rotational angles

  // mouse control for two angles
  //  angRx += 0.05 * (_root._xmouse-275)/550;
  //  ang2 += 0.05 * (_root._ymouse-200)/400;

  // keyboard control model
  //trace("g_iControl="+g_iControl);
  if(g_iControl == KEYBOARD){
    iKeySpeed = mcControlPanel.sldKeySpeed.GetValue();

    if(Key.isDown(keycodeA)){
      angRx += iKeySpeed;
    } else if(Key.isDown(keycodeZ)){
      angRx -= iKeySpeed;
    }
    if(Key.isDown(keycodeS)){
      angRy += iKeySpeed;
    } else if(Key.isDown(keycodeX)){
      angRy -= iKeySpeed;
    }
    if(Key.isDown(keycodeD)){
      angRz += iKeySpeed;
    } else if(Key.isDown(keycodeC)){
      angRz -= iKeySpeed;
    }
    if(Key.isDown(keycodeV)){
      ang2 += iKeySpeed;
    } else if(Key.isDown(keycodeB)){
      ang2 -= iKeySpeed;
    }
    if(Key.isDown(keycodeN)){
      ang3 += iKeySpeed;
    } else if(Key.isDown(keycodeJ)){
      ang3 -= iKeySpeed;
    }
  } else if(g_iControl == AUTOPILOT){

    // slider control autopilot
    // radians wrap at PI (?)
    angRx += mcControlPanel.sldAngRx.GetValue();
    angRy += mcControlPanel.sldAngRy.GetValue();
    angRz += mcControlPanel.sldAngRz.GetValue();
    ang2 += mcControlPanel.slider2.GetValue();
    ang3 += mcControlPanel.slider3.GetValue();

  } else {
    trace("unknown control mode "+g_iControl);
  }

  // update control display
  angRxModPI = angRx % twoPI;
  angRyModPI = angRy % twoPI;
  angRzModPI = angRz % twoPI;
  angModPI2 = ang2 % twoPI;
  angModPI3 = ang3 % twoPI;
  mcDialAngRx._rotation = angRxModPI * rad2deg;
  mcDialAngRy._rotation = angRyModPI * rad2deg;
  mcDialAngRz._rotation = angRzModPI * rad2deg;
  mcAngDial2._rotation = angModPI2 * rad2deg;
  mcAngDial3._rotation = angModPI3 * rad2deg;

  // Calculate points
  for(i = 0; i < points.length; i++){
    // each point has 4 coordinates
    x = points[i][0];
    y = points[i][1];
    z = points[i][2];
    w = points[i][3];

    // angRy is 1D rotation around y (changing x and z)
    y = y;
    z1 = z;
    z = Math.cos(angRy)*z1 + Math.sin(angRy)*x;
    x = -Math.sin(angRy)*z1 + Math.cos(angRy)*x;
    w = w;

    // angRx is 1D rotation around x (changing y and z)
    x = x;
    y1 = y;
    y = Math.cos(angRx)*y1 + Math.sin(angRx)*z;
    z = -Math.sin(angRx)*y1 + Math.cos(angRx)*z;
    w = w;

    // angRz is 1D rotation around z (changing y and z)
    y1 = y;
    y = Math.cos(angRz)*y1 + Math.sin(angRz)*x;
    x = -Math.sin(angRz)*y1 + Math.cos(angRz)*x;
    z = z;
    w = w;

    // Ang 2 is rotation around ? - changing x and w
    x = x*Math.cos(ang2) + w*Math.sin(ang2);
    y = y;
    z = z;
    w = -x*Math.sin(ang2)+w*Math.cos(ang2);

    // Ang 3 is rotation around ? - changing y
    x = x ;
    y = y*Math.cos(ang3) - w*Math.sin(ang3);
    z = z;

    // fake 3d depth in 2d
    scalar = zScale / (z + zScale );

    // plot 2d points
    _root["point"+i]._x = scalar *x *xyScale +xoff;
    _root["point"+i]._y = scalar *y *xyScale +yoff;
  }

  for(i = 0; i < faces.length; i++){
    var d = _root["face"+i];
    d.clear();
    if(showFaces){
      d.moveTo(_root["point"+faces[i][0]]._x,_root["point"+faces[i][0]]._y);

      // generate face colours from i
      //col = 0x006699 + (i * 0x000080 - 0x000048) + (-i * 0x008000 - 0x004800);

      // face colours from xml
      col = faceColours[i];

      d.beginFill(col,alpha);
      for(j = 1; j < faces[i].length; j++){
        var c = _root["point"+faces[i][j]];
        d.lineTo(c._x, c._y);
      }
      d.endFill();
    }
  }

  _root.line.clear();
  var d = _root.line;
  if(showLines){
    for(i = 0; i < lines.length; i++){
      d.moveTo(_root["point"+lines[i][0]]._x,_root["point"+lines[i][0]]._y);
      d.lineStyle( 1, 0xcccccc, 100 );
      for(j = 1; j < lines[i].length; j++){
        d.lineTo(_root["point"+lines[i][j]]._x,_root["point"+lines[i][j]]._y);
      }
    }
  }
}


function createAssets(){

  for(i = 0; i < points.length; i++){
    _root.createEmptyMovieClip("point"+i,i);
    // colours from xml
    col = pointColours[i];
    if(showPoints){
      with(_root["point"+i]){
        beginFill(col,100);
        moveTo(-2,-2)
        lineTo(-2,2);
        lineTo(2,2);
        lineTo(2,-2);
        endFill();
      }
    }
  }

  for(j = 0; j < faces.length; j++){
    _root.createEmptyMovieClip("face"+j, 600 + j);
  }

  _root.createEmptyMovieClip("line", 200);

  int1 = setInterval(paint, 15);
}

function parseXML(){
  var c = file.firstchild.firstChild;
  points = new Array();
  pointColours = new Array();
  for(i = 0; i < c.childNodes.length; i++){
    points.push( c.childNodes[i].attributes.coords.split(",") );
    pointColours.push( c.childNodes[i].attributes.colour );
  }

  var c = file.firstchild.childNodes[1];
  lines = new Array();
  for(i = 0; i < c.childNodes.length; i++){
    lines.push( c.childNodes[i].attributes.points.split(",") );
  }

  var c = file.firstchild.childNodes[2];
  faces = new Array();
  faceColours = new Array();
  for(i = 0; i < c.childNodes.length; i++){
    faces.push( c.childNodes[i].attributes.points.split(",") );
    faceColours.push( c.childNodes[i].attributes.colour );
  }
  createAssets();
}

function xmlLoaded(success){
  if(success){
    parseXML();
  }
}

function toggleLines(){
  showLines = !showLines;
}

function togglePoints(){
  showPoints = !showPoints;
}

function toggleFaces(){
  showFaces = !showFaces;
}

showLines = true;
showPoints = true;
showFaces = true;


//Define points
file = new XML();
file.ignoreWhite = true;
file.onLoad = xmlLoaded;

file.load("hypercube.xml");
//file.load("hyperpyramid.xml");
